! ESA (C) 2000-2024 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!+
! 
!    Package        : epauxcheck
!
!    Creation date  : 27-Jun-2024
!
!    Description    : Check PNAUX file for CCD pn chain
!
!    Authors        :
!
!    
!		Michael Freyberg - MPE - mjf@mpe.mpg.de
!		
!
!-

module epauxcheck_mod

  use dal
  use param  
  use errorHandling

  implicit none 
  
  private 

  public epauxcheck

contains 
  
  !---------------------------------------------------------------------
  subroutine epauxcheck 
  !---------------------------------------------------------------------

  implicit none
  character(len=13), parameter :: CodeVersion = '0.2'
  character(len=1024)          :: fileName, auxfileName
  logical(kind=bool)           :: withpnauxname
  logical(kind=bool)           :: hasPNAUX1, hasPNAUX2
  logical(kind=bool)           :: noPNAUX1, noPNAUX2
  integer(kind=Int32)          :: ntaux1, ntaux2, nffff, nover, nloss
  character(len=40)            :: auxName
  character(len=22)            :: odf_icd

  integer(kind=Int16), dimension(:), pointer :: aux1ftcoarseCol => null()  
  integer(kind=Int16), dimension(:), pointer :: aux1ftfineCol => null()  
  integer(kind=Int16), dimension(:), pointer :: aux2NaboveCol => null()  
  integer(kind=Int16), dimension(:), pointer :: aux2NdefaCol => null()  
  type(DataSetT)               :: auxfile
  type(TableT)                 :: aux1table,aux2table
    

  write(errstr,*) 'EPAUXCHECK ',trim(CodeVersion),' is running...'
  call message(AppMsg,Sparse)

  withpnauxname    = booleanParameter("withpnauxname")
  auxfileName      = trim(stringParameter("pnauxname"))


!---------------------------------------------------------------------------
! get name of AUXFILE from ODF, open file and check existence of extensions
!
  auxName = trim(auxfileName(index(auxfileName, '/', .true.)+1:))
  auxfile = dataSet(auxfileName,READ,USE_ENVIRONMENT)

  write(errstr,*)' PNAUX file for check: ',trim(auxName)
  call message(AppMsg,Verbose)

!---------------------------------------------------------------------------
  hasPNAUX1 = hasBlock(auxfile,"PNAUX1")
  if (hasPNAUX1) then
    aux1table = table(auxfile,"PNAUX1")
    ntaux1 = numberOfRows(aux1table)
    write(errstr,*)' PNAUX1 has ',ntaux1,' number of rows'
    call message(AppMsg,Noisy)           ! number of rows (1..ntaux1)
  else                                   ! should never occur [who knows]
    ntaux1 = 0                           ! no rows
    call warning("noPNAUX1","AUX file does not contain extension PNAUX1.")
    noPNAUX1 = .true.
    write(errstr,*) 'The ODF [',ODF_icd,'] is invalid, please contact the originator of this data.'
    call warning('illegalODF')
    write(errstr,*) 'Several corrections cannot be applied in subsequent tasks. This may cause follow-up error messages.'
    call message(AppMsg,Sparse)
  endif


!---------------------------------------------------------------------------
  hasPNAUX2 = hasBlock(auxfile,"PNAUX2")
  if (hasPNAUX2) then
    aux2table = table(auxfile,"PNAUX2")
    ntaux2 = numberOfRows(aux2table)
    write(errstr,*)' PNAUX2 has ',ntaux2,' number of rows'
    call message(AppMsg,Noisy)           ! number of rows (1..ntaux2)
  else
    ntaux2 = 0                           ! no rows
    call warning("noPNAUX2","AUX file does not contain extension PNAUX2.")
    noPNAUX2 = .true.
    write(errstr,*) 'The ODF [',ODF_icd,'] is invalid, please contact the originator of this data.'
    call warning('illegalODF')
    write(errstr,*) 'Several corrections cannot be applied in subsequent tasks. This may cause follow-up error messages.'
    call message(AppMsg,Sparse)
  endif

!---------------------------------------------------------------------------
  if (ntaux1 > 0) then
    aux1table = table(auxfile,'PNAUX1')
    aux1FtcoarseCol => int16Data(column(aux1table,"FTCOARSE",READ))
    aux1FtfineCol => uint16Data(column(aux1table,"FTFINE",READ))
    nffff = COUNT(MASK = aux1FtcoarseCol(0:ntaux1-1) == 32767)
    write(errstr,*)' PNAUX1 has ',nffff,' rows with FTCOARSE == 32767'
    call message(AppMsg,Verbose)
    call release(aux1table)
  endif

  if (ntaux1 > 2) then
    aux2table = table(auxfile,'PNAUX2')
    aux2NaboveCol => uint16Data(column(aux2table,"NABOVE",READ))
    aux2NdefaCol => int16Data(column(aux2table,"NDEFA",READ))
    nloss = COUNT(MASK = aux2NaboveCol(0:ntaux2-1)-aux2NdefaCol(0:ntaux2-1) > 0)
    write(errstr,*)' PNAUX2 has ',nloss,' rows with NDEFA < NABOVE'
    call message(AppMsg,Verbose)
    nover = COUNT(MASK = aux2NaboveCol(0:ntaux2-1)-aux2NdefaCol(0:ntaux2-1) > 511)
    write(errstr,*)' PNAUX2 has ',nover,' rows with NABOVE - NDEFA > 511'
    call message(AppMsg,Verbose)
    call release(aux2table)
  endif

  end subroutine  epauxcheck
  
end module epauxcheck_mod
